import { defineComponent, ref, isRef, unref, isReactive, toRaw, onMounted, markRaw, reactive, computed, h } from 'vue';
import { Col, Row, Form, FormItem } from 'ant-design-vue';

var __defProp$1 = Object.defineProperty;
var __getOwnPropSymbols$1 = Object.getOwnPropertySymbols;
var __hasOwnProp$1 = Object.prototype.hasOwnProperty;
var __propIsEnum$1 = Object.prototype.propertyIsEnumerable;
var __defNormalProp$1 = (obj, key, value) => key in obj ? __defProp$1(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues$1 = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp$1.call(b, prop))
      __defNormalProp$1(a, prop, b[prop]);
  if (__getOwnPropSymbols$1)
    for (var prop of __getOwnPropSymbols$1(b)) {
      if (__propIsEnum$1.call(b, prop))
        __defNormalProp$1(a, prop, b[prop]);
    }
  return a;
};
const add = (comp, flag, source, key) => {
  if (!key)
    return;
  const idx = source.value.findIndex((p) => p.elementKey === key);
  if (idx <= -1) {
    console.warn(`key\u4E0D\u5B58\u5728:add${flag}`);
  }
  const breakPoint = flag === "after" ? idx + 1 : idx;
  const left = source.value.slice(0, breakPoint);
  const right = source.value.slice(breakPoint, source.value.length);
  source.value = [...left, ...comp, ...right];
};
const setPropsInner = (props, source, key) => {
  const idx = source.value.findIndex((item) => item.elementKey === key);
  if (idx <= -1) {
    console.warn("key\u4E0D\u5B58\u5728:setProps");
    return;
  }
  const { props: oldProps } = source.value[idx];
  source.value[idx].props = __spreadValues$1(__spreadValues$1({}, oldProps || {}), props);
};
const toggle = (hidden, key, source, model) => {
  const idx = source.value.findIndex((item) => item.elementKey === key);
  if (idx <= -1) {
    console.warn(`key\u4E0D\u5B58\u5728:${hidden ? "hide" : "show"}`);
    return;
  }
  source.value[idx].hidden = hidden;
  if (hidden) {
    delete model[key];
  } else {
    model[key] = void 0;
  }
};

var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var componentsRender = defineComponent({
  name: "JRender",
  props: {
    components: {
      type: Object,
      default: []
    },
    formProps: {
      type: Object,
      default: void 0
    },
    gutter: {
      type: Object,
      default: [12, 12]
    },
    isForm: {
      type: Boolean,
      default: true
    },
    status: {
      type: String,
      default: ""
    }
  },
  emits: ["JRmounted"],
  setup(props, context) {
    const originModel = {};
    const form = ref();
    let unwrap = props.components;
    const cloneComponents = unwrap.map((item) => __spreadValues({}, item));
    if (isRef(unwrap))
      unwrap = unref(unwrap);
    if (isReactive(unwrap))
      unwrap = toRaw(unwrap);
    const components = ref(unwrap.map((item) => {
      const { element, label, elementKey, props: p, action, hidden, span, children, type, defaultValue, rules, mounted } = item;
      if (elementKey)
        originModel[elementKey] = defaultValue;
      if (mounted)
        onMounted(() => mounted(item));
      const rawElement = typeof element !== "string" ? markRaw(element) : element;
      const rawChildren = typeof children !== "string" && children ? markRaw(children) : children;
      return { element: rawElement, children: rawChildren, action, elementKey, span, type, defaultValue, label, props: p, hidden, rules };
    }));
    const { expose } = context;
    const model = reactive(originModel);
    const addBefore = (key, comp) => add(comp, "before", components, key);
    const addAfter = (key, comp) => add(comp, "after", components, key);
    const hide = (key) => toggle(true, key, components, model);
    const show = (key) => toggle(false, key, components, model);
    const setProps = (props2, key) => setPropsInner(props2, components, key);
    const toExpose = { form, model, addAfter, addBefore, setProps, show, hide, components, cloneComponents };
    expose(toExpose);
    const renderElement = (comp) => {
      const { action, element, children } = comp;
      const resolvedEvents = action ? action(toExpose, props, context) : {};
      const placeholderPrefix = comp.type ? "\u8BF7\u9009\u62E9" : "\u8BF7\u8F93\u5165";
      const vmodel = { "onUpdate:value": (val) => model[comp.elementKey] = val, value: model[comp.elementKey] };
      if (comp.props)
        comp.props.placeholder = comp.props.placeholder || `${placeholderPrefix}${comp.label}`;
      return h(element, __spreadValues(__spreadValues(__spreadValues({}, comp.props), resolvedEvents), vmodel), () => children);
    };
    const renderFormItem = (component) => {
      const { label, rules, elementKey } = component;
      return h(FormItem, { rules, label, name: elementKey }, () => renderElement(component));
    };
    const { isForm } = props;
    const cols = computed(() => components.value.filter(({ hidden }) => hidden !== true).map((item) => h(Col, { span: item.span }, () => isForm ? renderFormItem(item) : renderElement(item))));
    const row = h(Row, { gutter: props.gutter }, () => cols.value);
    onMounted(() => context.emit("JRmounted"));
    return () => isForm ? h(Form, __spreadValues({ model, ref: form }, props.formProps), row) : h("div", {}, row);
  }
});

export { componentsRender as JRender };
